#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#ifdef __BORLANDC__
#pragma hdrstop
#endif //__BORLANDC__

#include <wx/app.h>
#include <wx/menu.h>
#include <wx/sizer.h>
#include <wx/panel.h>
#include <wx/listbox.h>
#include <wx/button.h>
#include <wx/splitter.h>
#include <wx/checktree.h>
#include <wx/frame.h>
#include <wx/msgdlg.h>

///////////////////////////////////////////////////////////////////////////

#define idMenuQuit 1000
#define idMenuAbout 1001
#define wxID_CHECKBUTTON 1002
#define wxID_UNCHECKBUTTON 1003
#define wxID_ENABLEBUTTON 1004
#define wxID_DISABLEBUTTON 1005

///////////////////////////////////////////////////////////////////////////////
/// Class myFrame
///////////////////////////////////////////////////////////////////////////////
class myFrame : public wxFrame
{
	private:

	protected:
		wxMenuBar* mbar;
		wxMenu* fileMenu;
		wxMenu* helpMenu;
		wxStatusBar* statusBar;
		wxSplitterWindow* m_splitter1;
		wxPanel* m_panel1;
		wxPanel* m_panel3;
		wxPanel* m_panel4;
		wxCheckTree* m_treeCtrl1;
		wxPanel* m_panel6;
		wxTextCtrl* m_textCtrl1;
		wxListBox* m_listBox1;
		wxButton* m_button1;
		wxButton* m_button2;
		wxButton* m_button3;
		wxButton* m_button4;

		//event handlers
		void OnClose( wxCloseEvent& event ) ;
		void OnQuit( wxCommandEvent& event );
		void OnAbout( wxCommandEvent& event );
		void OnFocusLost( wxFocusEvent& event );
		void On_Button( wxCommandEvent& event );

        void On_checktree_CHOICE(wxTreeEvent& event);
        void On_checktree_FOCUS(wxTreeEvent& event);

        //helper functions
        void build_tree();
        wxTreeItemId tree_add(const wxTreeItemId& parent, const wxString& text, int tree_node, bool set_state=false, bool checked=false, bool enabled=true );

        //private data
        std::map<int,wxTreeItemId> m_items;



	public:

		myFrame( wxWindow* parent, wxWindowID id = wxID_ANY, const wxString& title = wxT("wxCheckTree Sample Application"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize( 481,466 ), long style = wxDEFAULT_FRAME_STYLE|wxTAB_TRAVERSAL );

		~myFrame();

		void m_splitter1OnIdle( wxIdleEvent& )
		{
			m_splitter1->SetSashPosition( 161 );
			m_splitter1->Disconnect( wxEVT_IDLE, wxIdleEventHandler( myFrame::m_splitter1OnIdle ), NULL, this );
		}
};

myFrame::myFrame( wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size, long style ) : wxFrame( parent, id, title, pos, size, style )
{
	this->SetSizeHints( wxDefaultSize, wxDefaultSize );

	mbar = new wxMenuBar( 0 );
	fileMenu = new wxMenu();
	wxMenuItem* menuFileQuit;
	menuFileQuit = new wxMenuItem( fileMenu, idMenuQuit, wxString( wxT("&Quit") ) + wxT('\t') + wxT("Alt+F4"), wxT("Quit the application"), wxITEM_NORMAL );
	fileMenu->Append( menuFileQuit );

	mbar->Append( fileMenu, wxT("&File") );

	helpMenu = new wxMenu();
	wxMenuItem* menuHelpAbout;
	menuHelpAbout = new wxMenuItem( helpMenu, idMenuAbout, wxString( wxT("&About") ) + wxT('\t') + wxT("F1"), wxT("Show info about this application"), wxITEM_NORMAL );
	helpMenu->Append( menuHelpAbout );

	mbar->Append( helpMenu, wxT("&Help") );

	this->SetMenuBar( mbar );

	statusBar = this->CreateStatusBar( 2, wxST_SIZEGRIP, wxID_ANY );
	wxBoxSizer* bSizer1;
	bSizer1 = new wxBoxSizer( wxVERTICAL );

	m_splitter1 = new wxSplitterWindow( this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxSP_3D );
	m_splitter1->Connect( wxEVT_IDLE, wxIdleEventHandler( myFrame::m_splitter1OnIdle ), NULL, this );

	m_panel1 = new wxPanel( m_splitter1, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxSTATIC_BORDER|wxTAB_TRAVERSAL );
	wxBoxSizer* bSizer2;
	bSizer2 = new wxBoxSizer( wxVERTICAL );

	m_panel3 = new wxPanel( m_panel1, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL );
	wxBoxSizer* bSizer4;
	bSizer4 = new wxBoxSizer( wxVERTICAL );

	m_panel4 = new wxPanel( m_panel3, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL );
	wxBoxSizer* bSizer6;
	bSizer6 = new wxBoxSizer( wxVERTICAL );

	m_treeCtrl1 = new wxCheckTree( m_panel4, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTR_DEFAULT_STYLE|wxTR_HIDE_ROOT|wxNO_BORDER );
	bSizer6->Add( m_treeCtrl1, 1, wxALL|wxEXPAND, 1 );

    m_panel4->SetOwnBackgroundColour( m_treeCtrl1->GetBackgroundColour() );
	m_panel4->SetSizer( bSizer6 );
	m_panel4->Layout();
	bSizer6->Fit( m_panel4 );
	bSizer4->Add( m_panel4, 1, wxEXPAND | wxALL, 1 );

    m_panel3->SetOwnBackgroundColour( wxColor(127,157,185) );
	m_panel3->SetSizer( bSizer4 );
	m_panel3->Layout();
	bSizer4->Fit( m_panel3 );
	bSizer2->Add( m_panel3, 1, wxEXPAND | wxALL, 5 );


	m_panel1->SetSizer( bSizer2 );
	m_panel1->Layout();
	bSizer2->Fit( m_panel1 );
	m_panel6 = new wxPanel( m_splitter1, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxSTATIC_BORDER|wxTAB_TRAVERSAL );
	wxBoxSizer* bSizer7;
	bSizer7 = new wxBoxSizer( wxVERTICAL );

	m_textCtrl1 = new wxTextCtrl( m_panel6, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_DONTWRAP|wxTE_MULTILINE );
	bSizer7->Add( m_textCtrl1, 1, wxALL|wxEXPAND, 5 );

	wxBoxSizer* bSizer8;
	bSizer8 = new wxBoxSizer( wxHORIZONTAL );

	m_listBox1 = new wxListBox( m_panel6, wxID_ANY, wxDefaultPosition, wxDefaultSize, 0, NULL, 0 );
	bSizer8->Add( m_listBox1, 0, wxALL|wxEXPAND, 5 );

	wxBoxSizer* bSizer9;
	bSizer9 = new wxBoxSizer( wxVERTICAL );

	m_button1 = new wxButton( m_panel6, wxID_CHECKBUTTON, wxT("Check"), wxDefaultPosition, wxDefaultSize, 0 );
	bSizer9->Add( m_button1, 0, wxALL|wxALIGN_RIGHT, 5 );

	m_button2 = new wxButton( m_panel6, wxID_UNCHECKBUTTON, wxT("Uncheck"), wxDefaultPosition, wxDefaultSize, 0 );
	bSizer9->Add( m_button2, 0, wxALL|wxALIGN_RIGHT, 5 );

	m_button3 = new wxButton( m_panel6, wxID_ENABLEBUTTON, wxT("Enable"), wxDefaultPosition, wxDefaultSize, 0 );
	bSizer9->Add( m_button3, 0, wxALL|wxALIGN_RIGHT, 5 );

	m_button4 = new wxButton( m_panel6, wxID_DISABLEBUTTON, wxT("Disable"), wxDefaultPosition, wxDefaultSize, 0 );
	bSizer9->Add( m_button4, 0, wxALL|wxALIGN_RIGHT, 5 );


	bSizer8->Add( bSizer9, 0, wxALIGN_BOTTOM, 5 );


	bSizer7->Add( bSizer8, 0, wxEXPAND, 5 );


	m_panel6->SetSizer( bSizer7 );
	m_panel6->Layout();
	bSizer7->Fit( m_panel6 );
	m_splitter1->SplitVertically( m_panel1, m_panel6, 161 );
	bSizer1->Add( m_splitter1, 1, wxEXPAND, 5 );


	this->SetSizer( bSizer1 );
	this->Layout();

	// Connect Events
	this->Connect( wxEVT_CLOSE_WINDOW, wxCloseEventHandler( myFrame::OnClose ) );
	this->Connect( menuFileQuit->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( myFrame::OnQuit ) );
	this->Connect( menuHelpAbout->GetId(), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( myFrame::OnAbout ) );
	m_treeCtrl1->Connect( wxEVT_KILL_FOCUS, wxFocusEventHandler( myFrame::OnFocusLost ), NULL, this );
	m_button1->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( myFrame::On_Button ), NULL, this );
	m_button2->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( myFrame::On_Button ), NULL, this );
	m_button3->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( myFrame::On_Button ), NULL, this );
	m_button4->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( myFrame::On_Button ), NULL, this );

    m_treeCtrl1->Connect( wxEVT_CHECKTREE_CHOICE, wxTreeEventHandler( myFrame::On_checktree_CHOICE ), NULL, this );
    m_treeCtrl1->Connect( wxEVT_CHECKTREE_FOCUS, wxTreeEventHandler( myFrame::On_checktree_FOCUS ), NULL, this );

    build_tree();
}

myFrame::~myFrame()
{
	// Disconnect Events
	this->Disconnect( wxEVT_CLOSE_WINDOW, wxCloseEventHandler( myFrame::OnClose ) );
	this->Disconnect( idMenuQuit, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( myFrame::OnQuit ) );
	this->Disconnect( idMenuAbout, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( myFrame::OnAbout ) );
	m_treeCtrl1->Disconnect( wxEVT_KILL_FOCUS, wxFocusEventHandler( myFrame::OnFocusLost ), NULL, this );
	m_button1->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( myFrame::On_Button ), NULL, this );
	m_button2->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( myFrame::On_Button ), NULL, this );
	m_button3->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( myFrame::On_Button ), NULL, this );
	m_button4->Disconnect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( myFrame::On_Button ), NULL, this );
    m_treeCtrl1->Disconnect( wxEVT_CHECKTREE_CHOICE, wxTreeEventHandler( myFrame::On_checktree_CHOICE ), NULL, this );
    m_treeCtrl1->Disconnect( wxEVT_CHECKTREE_FOCUS, wxTreeEventHandler( myFrame::On_checktree_FOCUS ), NULL, this );

	m_items.clear();
}

void myFrame::OnClose(wxCloseEvent &event)
{
    Destroy();
}

void myFrame::OnQuit(wxCommandEvent &event)
{
    Destroy();
}

void myFrame::OnAbout(wxCommandEvent &event)
{
//    wxString msg = wxbuildinfo(long_f);
    wxMessageBox("wxCheckTree Demo", _("Welcome to..."));
}

void myFrame::On_checktree_CHOICE(wxTreeEvent& event)
{
    wxTreeItemId id=event.GetItem ();
    long l=event.GetExtraLong();

    (*m_textCtrl1) << "wxEVT_CHECKTREE_CHOICE detected: " <<m_treeCtrl1->GetItemText(id) << " has been " << (l?"checked":"unchecked") << "\n";
}

void myFrame::On_checktree_FOCUS(wxTreeEvent& event)
{
    (*m_textCtrl1) << "wxEVT_CHECKTREE_FOCUS detected.\n";

    m_panel4->SetOwnBackgroundColour(wxColor(176,194,211));
    m_panel4->ClearBackground();
}

void myFrame::OnFocusLost( wxFocusEvent& event )
{
    m_panel4->SetOwnBackgroundColour( m_treeCtrl1->GetBackgroundColour() );
    m_panel4->ClearBackground();
    event.Skip();
}

void myFrame::On_Button( wxCommandEvent& event )
{
    int i=m_listBox1->GetSelection();

    if(i==wxNOT_FOUND)
    {
        return;
    }

    i=reinterpret_cast<int>(m_listBox1->GetClientData(i));

    std::map<int,wxTreeItemId>::iterator it=m_items.find(i);

    if(it!=m_items.end())
    {
        if( event.GetId()==wxID_CHECKBUTTON )
        {
            m_treeCtrl1->Check(it->second,true);
        }
        else if( event.GetId()==wxID_UNCHECKBUTTON )
        {
            m_treeCtrl1->Uncheck(it->second);
        }
        else if( event.GetId()==wxID_ENABLEBUTTON )
        {
            m_treeCtrl1->EnableCheckBox(it->second,true);
        }
        else if( event.GetId()==wxID_DISABLEBUTTON )
        {
            m_treeCtrl1->DisableCheckBox(it->second);
        }
    }
}

wxTreeItemId myFrame::tree_add(const wxTreeItemId& parent, const wxString& text, int tree_node, bool set_state, bool checked, bool enabled )
{
    wxTreeItemId id=m_treeCtrl1->AppendItem(parent,text);
    m_items.insert(std::pair<int,wxTreeItemId>(tree_node,id));

    if(set_state)
    {
        m_treeCtrl1->MakeCheckable(id, checked);
        m_treeCtrl1->EnableCheckBox(id, enabled);
    }

    m_listBox1->Append(text,reinterpret_cast<void*>(tree_node));

    return id;
}

void myFrame::build_tree()
{
    wxTreeItemId id2=m_treeCtrl1->AddRoot("Root");
    wxTreeItemId card=m_treeCtrl1->AppendItem(id2,"Cardinal Units");
    wxTreeItemId weight=m_treeCtrl1->AppendItem(id2,"Weight Units");
    wxTreeItemId vol=m_treeCtrl1->AppendItem(id2,"Volume Units");
    wxTreeItemId weight_stan=m_treeCtrl1->AppendItem(weight,"Standard");
    wxTreeItemId weight_met=m_treeCtrl1->AppendItem(weight,"Metric");
    wxTreeItemId vol_stan=m_treeCtrl1->AppendItem(vol,"Standard");
    wxTreeItemId vol_met=m_treeCtrl1->AppendItem(vol,"Metric");

    tree_add(card,"Count",0,true,true);

    tree_add(weight,"With Count",1,true,false,false);
    tree_add(weight_stan,"ounce",2,true);
    tree_add(weight_stan,"pound",3,true);
    id2=tree_add(weight_met,"gram",4,true);
    tree_add(id2,"100 grams",5,true);
    tree_add(weight_met,"kilogram",6,true);

    id2=tree_add(vol_stan,"tsp",7,true);
    tree_add(id2,"1/4 tsp",8,true);
    tree_add(id2,"1/3 tsp",9,true);
    tree_add(id2,"1/2 tsp",10,true);

    id2=tree_add(vol_stan,"tbsp",11,true);
    tree_add(id2,"1/4 tbsp",12,true);
    tree_add(id2,"1/3 tbsp",13,true);
    tree_add(id2,"1/2 tbsp",14,true);

    tree_add(vol_stan,"fl oz",19,true);

    id2=tree_add(vol_stan,"cup",15,true);
    tree_add(id2,"1/4 cup",16,true);
    tree_add(id2,"1/3 cup",17,true);
    tree_add(id2,"1/2 cup",18,true);

    tree_add(vol_stan,"pint",20,true);
    tree_add(vol_stan,"quart",21,true);
    tree_add(vol_stan,"gallon",22,true);

    id2=tree_add(vol_met,"milliliter",23,true);
    tree_add(id2,"100 milliliters",24,true);
    tree_add(id2,"250 milliliters",25,true);
    tree_add(vol_met,"liter",26,true);
}

//application
class myApp : public wxApp
{
    public:
        bool OnInit()
        {
            myFrame* frame = new myFrame(0L);
            frame->Show();

            return true;
        }
};

IMPLEMENT_APP(myApp);

